/******************************************************************************
 *
 * Header file for harness forker (forker process) module.
 * Copyright (c) 2002-2006 Oak Ridge National Laboratory.
 *
 * For more information see the following files in the source distribution top-
 * level directory or package data directory (usually /usr/local/share/package):
 *
 * - README    for general package information.
 * - INSTALL   for package install information.
 * - COPYING   for package license information and copying conditions.
 * - AUTHORS   for package authors information.
 * - ChangeLog for package changes information.
 *
 ******************************************************************************/

/* Avoid to include the content of this header file twice. */
#ifndef HARNESS_FORKER_H
#define HARNESS_FORKER_H

/******************************************************************************
 *
 * Macros
 *
 ******************************************************************************/

/*
 * Forker module defaults.
 */
#define HARNESS_FORKER_BUFFER_SIZE 4096 /* buffer size */

/******************************************************************************
 *
 * Includes
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Data Types
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Function Prototypes
 *
 ******************************************************************************/

/*
 * Sets the forker process buffer size.
 * This function may only be called before initializing.
 *
 * size   = The forker process buffer size.
 * return = 0 on success or -1 on any error with errno set appropriately.
 */
int harness_forker_size_set (unsigned int size);

/*
 * Increases/Decreases the forker process buffer size.
 * This function may only be called before initializing.
 *
 * size   = The forker process buffer size increment/decrement.
 * return = 0 on success or -1 on any error with errno set appropriately.
 */
int harness_forker_size_add (int size);

/*
 * Gets the forker process buffer size.
 *
 * size   = The forker process buffer size (return).
 * return = 0 on success or -1 on any error with errno set appropriately.
 */
int harness_forker_size_get (unsigned int *size);

/******************************************************************************
 *
 * Data
 *
 ******************************************************************************/

/* None at this time. */

#endif /* HARNESS_FORKER_H */

/******************************************************************************
 *
 * END OF FILE
 *
 ******************************************************************************/
