/******************************************************************************
 *
 * Header file for harness kernel (kernel process) module.
 * Copyright (c) 2002-2006 Oak Ridge National Laboratory.
 *
 * For more information see the following files in the source distribution top-
 * level directory or package data directory (usually /usr/local/share/package):
 *
 * - README    for general package information.
 * - INSTALL   for package install information.
 * - COPYING   for package license information and copying conditions.
 * - AUTHORS   for package authors information.
 * - ChangeLog for package changes information.
 *
 ******************************************************************************/

/* Avoid to include the content of this header file twice. */
#ifndef HARNESS_KERNEL_H
#define HARNESS_KERNEL_H

/******************************************************************************
 *
 * Macros
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Includes
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Data Types
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Function Prototypes
 *
 ******************************************************************************/

/*
 * Sets the block flag.
 * This function may only be called before initializing.
 *
 * block   = The block flag to indicate wether to run the kernel in this
 *           thread and block until finished, or to create a new kernel
 *           thread and return after kernel startup.
 * return  = 0 on success or -1 on any error with errno set appropriately.
 */
int harness_kernel_block_set (unsigned int block);

/*
 * Sets the init options to be applied at startup.
 * This function may only be called before initializing.
 *
 * options = The init options to be applied at startup.
 *           A NULL terminated array of strings containing option/value
 *           pairs. Possible option/value pairs are:
 *           'load' and 'VALUE' to load a plug-in with the name of VALUE at
 *               start-up. Allows plug-in to load more than once and does 
 *               not export its symbols. VALUE cannot be ommitted.
 *           'load-unique' and 'VALUE' to load a plug-in with the name of
 *               VALUE at start-up. Allows plug-in to load only once and
 *               does not export its symbols. VALUE cannot be ommitted.
 *           'load-export' and 'VALUE' to load a plug-in with the name of
 *               VALUE at start-up. Allows plug-in to load only once and
 *               exports its symbols. VALUE cannot be ommitted.
 *           'exec' and 'VALUE' to execute a program with the name of VALUE
 *               in a child process at start-up. VALUE cannot be ommitted.
 *           All options are applied in the order they appear.
 * return  = 0 on success or -1 on any error with errno set appropriately.
 */
int harness_kernel_options_set (char **options);

/*
 * Gets the block flag.
 *
 * block   = The block flag to indicate wether to run the kernel in this
 *           thread and block until finished, or to create a new kernel
 *           thread and return after kernel startup.
 * return  = 0 on success or -1 on any error with errno set appropriately.
 */
int harness_kernel_block_get (unsigned int *block) ;

/*
 * Gets the forker process buffer size.
 *
 * options = The init options to be applied at startup (return).
 *           A NULL terminated array of strings containing option/value
 *           pairs. Possible option/value pairs are:
 *           'load' and 'VALUE' to load a plug-in with the name of VALUE at
 *               start-up. Allows plug-in to load more than once and does 
 *               not export its symbols. VALUE cannot be ommitted.
 *           'load-unique' and 'VALUE' to load a plug-in with the name of
 *               VALUE at start-up. Allows plug-in to load only once and
 *               does not export its symbols. VALUE cannot be ommitted.
 *           'load-export' and 'VALUE' to load a plug-in with the name of
 *               VALUE at start-up. Allows plug-in to load only once and
 *               exports its symbols. VALUE cannot be ommitted.
 *           'exec' and 'VALUE' to execute a program with the name of VALUE
 *               in a child process at start-up. VALUE cannot be ommitted.
 *           All options are applied in the order they appear.
 * return  = 0 on success or -1 on any error with errno set appropriately.
 */
int harness_kernel_options_get (char ***options);

/*
 * Asynchonously shuts down the kernel and subsequently the forker.
 */
void harness_kernel_shutdown ();

/*
 * Waits for the kernel thread to shut down if started in non-blocking mode.
 *
 * return = 0 on success or -1 on any error with errno set appropriately.
 */
int harness_kernel_join ();

/******************************************************************************
 *
 * Data
 *
 ******************************************************************************/

/* None at this time. */

#endif /* HARNESS_KERNEL_H */

/******************************************************************************
 *
 * END OF FILE
 *
 ******************************************************************************/
