/******************************************************************************
 *
 * Header file for rmix invokeref (invocation reference) module.
 * Copyright (c) 2002-2006 Oak Ridge National Laboratory.
 *
 * For more information see the following files in the source distribution top-
 * level directory or package data directory (usually /usr/local/share/package):
 *
 * - README    for general package information.
 * - INSTALL   for package install information.
 * - COPYING   for package license information and copying conditions.
 * - AUTHORS   for package authors information.
 * - ChangeLog for package changes information.
 *
 ******************************************************************************/

/* Avoid to include the content of this header file twice. */
#ifndef RMIX_INVOKEREF_H
#define RMIX_INVOKEREF_H

/******************************************************************************
 *
 * Macros
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Includes
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Data Types
 *
 ******************************************************************************/

/* Invocation reference data type. */
typedef struct {
  unsigned int      invocation; /* invocation handle       */
  unsigned int      provider;   /* provider handle         */
  rmix_remoteref_t *remoteref;  /* remote object reference */
  rmix_interface_t *interface;  /* object interface        */
  unsigned int      method;     /* method index            */
} rmix_invokeref_t;

/******************************************************************************
 *
 * Function Prototypes
 *
 ******************************************************************************/

/*
 * Creates a invocation reference.
 *
 * invokeref  = The invocation reference to create (return).
 * invocation = The invocation handle.
 * provider   = The provider handle.
 * remoteref  = The remote object reference.
 * interface  = The object interface.
 * method     = The method index.
 * return     = 0 on success or -1 on any error with errno set appropriately.
 */
int rmix_invokeref_create (rmix_invokeref_t       **invokeref,
                           unsigned int             invocation,
                           unsigned int             provider,
                           const rmix_remoteref_t  *remoteref,
                           const rmix_interface_t  *interface,
                           unsigned int             method);

/*
 * Destroys a invocation reference.
 *
 * invokeref = The invocation reference to destroy (also return).
 * return    = 0 on success or -1 on any error with errno set appropriately.
 */
int rmix_invokeref_destroy (rmix_invokeref_t **invokeref);

/*
 * Duplicates an invocation reference.
 *
 * invokeref1 = The invocation reference to create (return).
 * invokeref2 = The invocation reference to duplicate.
 * return     = 0 on success or -1 on any error with errno set appropriately.
 */
int rmix_invokeref_duplicate (rmix_invokeref_t       **invokeref1,
                              const rmix_invokeref_t  *invokeref2);

/*
 * Compares two invocation references.
 *
 * diff       = The difference between both invocation references (return).
 *              diff > 0 if invokeref1 < invokeref2,
 *              diff < 0 if invokeref1 > invokeref2, and
 *              diff = 0 if invokeref1 = invokeref2
 * invokeref1 = The invocation reference to compare against.
 * invokeref2 = The invocation reference to compare with.
 * return     = 0 on success or -1 on any error with errno set appropriately.
 */
int rmix_invokeref_compare (int                    *diff,
                            const rmix_invokeref_t *invokeref1,
                            const rmix_invokeref_t *invokeref2);

/******************************************************************************
 *
 * Data
 *
 ******************************************************************************/

/* None at this time. */

#endif /* RMIX_INVOKEREF_H */

/******************************************************************************
 *
 * END OF FILE
 *
 ******************************************************************************/
