/******************************************************************************
 *
 * Header file for rmix objects (object registry) module.
 * Copyright (c) 2002-2006 Oak Ridge National Laboratory.
 *
 * For more information see the following files in the source distribution top-
 * level directory or package data directory (usually /usr/local/share/package):
 *
 * - README    for general package information.
 * - INSTALL   for package install information.
 * - COPYING   for package license information and copying conditions.
 * - AUTHORS   for package authors information.
 * - ChangeLog for package changes information.
 *
 ******************************************************************************/

/* Avoid to include the content of this header file twice. */
#ifndef RMIX_OBJECTS_H
#define RMIX_OBJECTS_H

/******************************************************************************
 *
 * Macros
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Includes
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Data Types
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Function Prototypes
 *
 ******************************************************************************/

/*
 * Adds an object to the object registry. The object may already exist in the
 * registry. In this case its reference counter is increased and the same
 * object id is returned. An object may register with a different interface. In
 * this case a new object id is returned.
 *
 * objectid  = The object id (return).
 * object    = The object to add (may be NULL).
 * interface = The object interface to add.
 * return    = 0 on success or -1 on any error with errno set appropriately.
 */
int rmix_objects_add (unsigned int           *objectid,
                      void                   *object,
                      const rmix_interface_t *interface);

/*
 * Adds an object to the object registry with a forced object id (if not already
 * used by another object). The object may already exist under this object id in
 * the registry. In this case its reference counter is increased. The
 * object interfaces must be equal for successful registration.
 *
 * objectid  = The object id.
 * object    = The object to add.
 * interface = The object interface to add.
 * return    = 0 on success or -1 on any error with errno set appropriately.
 */
int rmix_objects_add2 (unsigned int            objectid,
                       void                   *object,
                       const rmix_interface_t *interface);

/*
 * Removes an object from the object registry. If its reference counter is not
 * zero, the object will remain in the object registry.
 *
 * objectid = The object id to remove.
 * return   = 0 on success or -1 on any error with errno set appropriately.
 */
int rmix_objects_remove (unsigned int objectid);

/*
 * Returns the object and interface of a registered object.
 *
 * objectid  = The object id.
 * object    = The object (return).
 * interface = The object interface (return).
 * return    = 0 on success or -1 on any error with errno set appropriately.
 */
int rmix_objects_get (unsigned int             objectid,
                      void                   **object,
                      const rmix_interface_t **interface);

/*
 * Returns the object and interface of a registered object.
 *
 * remoteref = The object remote reference.
 * object    = The object (return).
 * interface = The object interface (return).
 * return    = 0 on success or -1 on any error with errno set appropriately.
 */
int rmix_objects_get2 (const rmix_remoteref_t  *remoteref,
                       void                   **object,
                       const rmix_interface_t **interface);

/******************************************************************************
 *
 * Data
 *
 ******************************************************************************/

/* None at this time. */

#endif /* RMIX_OBJECTS_H */

/******************************************************************************
 *
 * END OF FILE
 *
 ******************************************************************************/
