/******************************************************************************
 *
 * Source file for the RMIX parameters (export parameter list) module.
 * Copyright (c) 2002-2006 Oak Ridge National Laboratory.
 *
 * For more information see the following files in the source distribution top-
 * level directory or package data directory (usually /usr/local/share/package):
 *
 * - README    for general package information.
 * - INSTALL   for package install information.
 * - COPYING   for package license information and copying conditions.
 * - AUTHORS   for package authors information.
 * - ChangeLog for package changes information.
 *
 ******************************************************************************/

/******************************************************************************
 *
 * Macros
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Includes
 *
 ******************************************************************************/

/* Main module header file. */
#include "rmix.h"

/******************************************************************************
 *
 * Data Types
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Data
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Function Prototypes
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Functions
 *
 ******************************************************************************/

/*
 * Creates an export parameter list.
 *
 * parameters = The export parameter list to create (return).
 * count      = The export parameter list count.
 * array      = The export parameter list array.
 * return     = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_parameters_create"
int rmix_parameters_create (rmix_parameters_t      **parameters,
                            unsigned int             count,
                            const rmix_parameter_t **array) {
  unsigned int index;
#ifdef DEBUG
  /* Check parameters parameter. */
  if (NULL == parameters) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check array parameter. */
  if ((0 != count)&&(NULL == array)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(array parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check count parameter. */
  if ((0 == count)&&(NULL != array)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(count parameter is zero)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Allocate export parameter list. */
  if (NULL == (*parameters = malloc(sizeof(rmix_parameters_t)))) {
    int errno2 = errno;
    RMIX_LOG((RMIX_WARN(unable to allocate export parameter list)))
    errno = errno2;
    return -1;
  }
  (*parameters)->count = count;
  /* Check for empty export parameter list. */
  if (0 == (*parameters)->count) {
    int errno2 = errno;
    /* Set empty export parameter list array and return. */
    (*parameters)->array = NULL;
    errno = errno2;
    return 0;
  }
  /* Allocate export parameter list array. */
  if (NULL == ((*parameters)->array = malloc(count *
                                             sizeof(rmix_parameter_t*)))) {
    int errno2 = errno;
    RMIX_LOG((RMIX_WARN(unable to allocate export parameter list array)))
    /* Deallocate export parameter list. */
    free(*parameters);
    errno = errno2;
    return -1;
  }
  /* Duplicate export parameter list. */
  for (index = 0; index < count; index++) {
    /* Duplicate export parameter. */
    if (0 != rmix_parameter_duplicate(&(*parameters)->array[index],
                                                      array[index])) {
      int errno2 = errno;
      RMIX_LOG((RMIX_WARN(unable to duplicate export parameter)))
      /* Destroy export parameter list. */
      while (0 != index--) {
        /* Destroy export parameter. */
        if (0 != rmix_parameter_destroy(&(*parameters)->array[index])) {
          RMIX_LOG((RMIX_WARN(unable to destroy export parameter.)))
        }
      }
      /* Deallocate export parameter list array. */
      free((*parameters)->array);
      /* Deallocate export parameter list. */
      free(*parameters);
      errno = errno2;
      return -1;
    }
  }
  return 0;
}

/*
 * Creates an export parameter list.
 *
 * parameters = The export parameter list to create (return).
 * count      = The export parameter list count.
 * names      = The export parameter list names array.
 * values     = The export parameter list values array.
 * return     = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_parameters_create2"
int rmix_parameters_create2 (rmix_parameters_t **parameters,
                             unsigned int        count,
                             const char        **names,
                             const char        **values) {
  unsigned int index;
#ifdef DEBUG
  /* Check parameters parameter. */
  if (NULL == parameters) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check names parameter. */
  if ((0 != count)&&(NULL == names)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(names parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check count parameter. */
  if ((0 == count)&&(NULL != names)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(count parameter is zero)))
    errno = EINVAL;
    return -1;
  }
  /* Check values parameter. */
  if ((0 != count)&&(NULL == values)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(values parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check count parameter. */
  if ((0 == count)&&(NULL != values)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(count parameter is zero)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Allocate export parameter list. */
  if (NULL == (*parameters = malloc(sizeof(rmix_parameters_t)))) {
    int errno2 = errno;
    RMIX_LOG((RMIX_WARN(unable to allocate export parameter list)))
    errno = errno2;
    return -1;
  }
  (*parameters)->count = count;
  /* Check for empty export parameter list. */
  if (0 == (*parameters)->count) {
    /* Set empty export parameter list array and return. */
    (*parameters)->array = NULL;
    return 0;
  }
  /* Allocate export parameter list array. */
  if (NULL == ((*parameters)->array = malloc(count *
                                             sizeof(rmix_parameter_t*)))) {
    int errno2 = errno;
    RMIX_LOG((RMIX_WARN(unable to allocate export parameter list array)))
    /* Deallocate export parameter list. */
    free(*parameters);
    errno = errno2;
    return -1;
  }
  /* Duplicate export parameter list. */
  for (index = 0; index < count; index++) {
    /* Duplicate export parameter. */
    if (0 != rmix_parameter_create(&(*parameters)->array[index],
                                   names[index], values[index])) {
      int errno2 = errno;
      RMIX_LOG((RMIX_WARN(unable to duplicate export parameter)))
      /* Destroy export parameter list. */
      while (0 != index--) {
        /* Destroy export parameter. */
        if (0 != rmix_parameter_destroy(&(*parameters)->array[index])) {
          RMIX_LOG((RMIX_WARN(unable to destroy export parameter.)))
        }
      }
      /* Deallocate export parameter list array. */
      free((*parameters)->array);
      /* Deallocate export parameter list. */
      free(*parameters);
      errno = errno2;
      return -1;
    }
  }
  return 0;
}

/*
 * Creates an export parameter list.
 *
 * parameters = The export parameter list to create (return).
 * count      = The export parameter list count.
 * pairs      = The export parameter list name-value pairs array. Each pair
 *              must be in the form <name>=<value>. Use a leading '\' for '='
 *              that are part of name or value and for other '\'s. For example,
 *              name\==\=value\\ means 'name=' and '=value\'.
 * return     = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_parameters_create3"
int rmix_parameters_create3 (rmix_parameters_t **parameters,
                             unsigned int        count,
                             const char        **pairs) {
  char         *name, *value;
  unsigned int  index;
#ifdef DEBUG
  /* Check parameters parameter. */
  if (NULL == parameters) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check pairs parameter. */
  if ((0 != count)&&(NULL == pairs)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(pairs parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check count parameter. */
  if ((0 == count)&&(NULL != pairs)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(count parameter is zero)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Allocate export parameter list. */
  if (NULL == (*parameters = malloc(sizeof(rmix_parameters_t)))) {
    int errno2 = errno;
    RMIX_LOG((RMIX_WARN(unable to allocate export parameter list)))
    errno = errno2;
    return -1;
  }
  (*parameters)->count = count;
  /* Check for empty export parameter list. */
  if (0 == (*parameters)->count) {
    /* Set empty export parameter list array and return. */
    (*parameters)->array = NULL;
    return 0;
  }
  /* Allocate export parameter list array. */
  if (NULL == ((*parameters)->array = malloc(count *
                                             sizeof(rmix_parameter_t*)))) {
    int errno2 = errno;
    RMIX_LOG((RMIX_WARN(unable to allocate export parameter list array)))
    /* Deallocate export parameter list. */
    free(*parameters);
    errno = errno2;
    return -1;
  }
  /* Duplicate export parameter list. */
  for (index = 0; index < count; index++) {
    /* Duplicate export parameter. */
    if (0 != rmix_parameter_create(&(*parameters)->array[index],
                                   pairs[index], pairs[index])) {
      int errno2 = errno;
      RMIX_LOG((RMIX_WARN(unable to duplicate export parameter)))
      /* Destroy export parameter list. */
      while (0 != index--) {
        /* Destroy export parameter. */
        if (0 != rmix_parameter_destroy(&(*parameters)->array[index])) {
          RMIX_LOG((RMIX_WARN(unable to destroy export parameter.)))
        }
      }
      /* Deallocate export parameter list array. */
      free((*parameters)->array);
      /* Deallocate export parameter list. */
      free(*parameters);
      errno = errno2;
      return -1;
    }
  }
  /* Separate export parameter name-value pairs and remove leading '\'s. */
  for (index = 0; index < count; index++) {
    /* Separate export parameter name-value pair
       and remove leading '\'s from name. */
    for (name = (*parameters)->array[index]->name; '\0' != name[0]; name++) {
      switch (name[0]) {
        /* Check for leading '\' in name. */
        case '\\': {
          /* Remove leading '\' in name. */
          memmove(name, name + 1, strlen(name));
          break;
        }
        /* Check for '=' separator in name. */
        case '=': {
          /* Separate name from name-value pair. */
          name[0] = '\0';
          name--;
          break;
        }
        default: {
          break;
        }
      }
    }
    /* Check for missing '=' separator. */
    if (strlen((*parameters)->array[index]->name ) ==
        strlen((*parameters)->array[index]->value)   ) {
      errno = EINVAL;
      RMIX_LOG((RMIX_WARN(missing '=' separator in name-value pair)))
      /* Destroy export parameter list. */
      for (index = 0; index < count; index++) {
        /* Destroy export parameter. */
        if (0 != rmix_parameter_destroy(&(*parameters)->array[index])) {
          RMIX_LOG((RMIX_WARN(unable to destroy export parameter.)))
        }
      }
      /* Deallocate export parameter list array. */
      free((*parameters)->array);
      /* Deallocate export parameter list. */
      free(*parameters);
      errno = EINVAL;
      return -1;
    }
    /* Separate value from name-value pair. */
    value = (*parameters)->array[index]->value;
    memcpy(value, name + 1, strlen(name + 1) + 1);
    /* Remove leading '\'s from value. */
    for (; '\0' != value[0]; value++) {
      /* Check for leading '\' in value. */
      if ('\\' == name[0]) {
        /* Remove leading '\' in value. */
        memmove(value, value + 1, strlen(value));
      }
    }
  }
  return 0;
}

/*
 * Creates an export parameter list.
 *
 * parameters = The export parameter list to create (return).
 * pairs      = The export parameter list name-value pairs string. Each pair
 *              must be in the form <name>=<value>. Use a leading '\' for '='
 *              and spaces that are part of name or value and for other '\'s.
 *              For example, 'name1\=\ =\=value1\\ name2\ =value2' means
 *              'name1= ' and '=value1\', and 'name 2' and 'value2'. The name-
 *              value pairs string may contain leading and trailing spaces, and
 *              multiple consecutive separating spaces.
 * return     = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_parameters_create4"
int rmix_parameters_create4 (rmix_parameters_t **parameters,
                             const char         *pairs) {
  char         *pairs2, *pairs3, *pair, *name, *value;
  unsigned int  count, index;
#ifdef DEBUG
  /* Check parameters parameter. */
  if (NULL == parameters) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Duplicate name-value pairs string. */
  if (NULL == (pairs2 = strdup(pairs))) {
    int errno2 = errno;
    RMIX_LOG((RMIX_WARN(unable to duplicate name-value pairs string)))
    errno = errno2;
    return -1;
  }
  /* Remove leading spaces. */
  for (pairs3 = pairs2; ' ' == pairs3[0]; pairs3++);
  if (pairs2 != pairs3) {
    memmove(pairs2, pairs3, strlen(pairs3) + 1);
  }
  /* Remove trailing spaces. */
  for (pairs3 = pairs2 + strlen(pairs3);
       (pairs3 != pairs2) && (' ' == pairs3[0]); pairs3[0] = '\0', pairs3--);
  /* Separate and count name-value pairs. */
  if ('\0' == pairs2[0]) {
    count = 0;
  } else {
    for (pairs3 = pairs2, count = 1; '\0' != pairs3[0]; pairs3++) {
      /* Check for leading '\' and separating space. */
      switch (pairs3[0]) {
        /* Check for leading '\'. */
        case '\\': {
          /* Check for following space. */
          if (' ' == pairs3[1]) {
            /* Remove leading '\'. */
            memmove(pairs3, pairs3 + 1, strlen(pairs3));
          } else {
           /* Ignore leading '\'. */
            pairs3++;
          }
          break;
        }
        /* Check for separating space. */
        case ' ': {
          /* Terminate pair. */
          pairs3[0] = '\0';
          count++;
          /* Remove trailing spaces. */
          for (;' ' == pairs3[1]; pairs3++, pairs3[0] = '\0');
          break;
        }
        default: {
          break;
        }
      }
    }
  }
  /* Allocate export parameter list. */
  if (NULL == (*parameters = malloc(sizeof(rmix_parameters_t)))) {
    int errno2 = errno;
    RMIX_LOG((RMIX_WARN(unable to allocate export parameter list)))
    errno = errno2;
    return -1;
  }
  (*parameters)->count = count;
  /* Check for empty export parameter list. */
  if (0 == (*parameters)->count) {
    /* Set empty export parameter list array and return. */
    (*parameters)->array = NULL;
    return 0;
  }
  /* Allocate export parameter list array. */
  if (NULL == ((*parameters)->array = malloc(count *
                                             sizeof(rmix_parameter_t*)))) {
    int errno2 = errno;
    RMIX_LOG((RMIX_WARN(unable to allocate export parameter list array)))
    /* Deallocate export parameter list. */
    free(*parameters);
    errno = errno2;
    return -1;
  }
  /* Duplicate export parameter list. */
  for (index = 0, pair = pairs2; index < count; index++) {
    /* Duplicate export parameter. */
    if (0 != rmix_parameter_create(&(*parameters)->array[index], pair, pair)) {
      int errno2 = errno;
      RMIX_LOG((RMIX_WARN(unable to duplicate export parameter)))
      /* Destroy export parameter list. */
      while (0 != index--) {
        /* Destroy export parameter. */
        if (0 != rmix_parameter_destroy(&(*parameters)->array[index])) {
          RMIX_LOG((RMIX_WARN(unable to destroy export parameter.)))
        }
      }
      /* Deallocate export parameter list array. */
      free((*parameters)->array);
      /* Deallocate export parameter list. */
      free(*parameters);
      errno = errno2;
      return -1;
    }
    /* Find next pair. */
    pair += strlen(pair) + 1;
    for (;'\0' == pair[0]; pair++);
  }
  /* Separate export parameter name-value pairs and remove leading '\'s. */
  for (index = 0; index < count; index++) {
    /* Separate export parameter name-value pair
       and remove leading '\'s from name. */
    for (name = (*parameters)->array[index]->name; '\0' != name[0]; name++) {
      switch (name[0]) {
        /* Check for leading '\' in name. */
        case '\\': {
          /* Remove leading '\' in name. */
          memmove(name, name + 1, strlen(name));
          break;
        }
        /* Check for '=' separator in name. */
        case '=': {
          /* Separate name from name-value pair. */
          name[0] = '\0';
          name--;
          break;
        }
        default: {
          break;
        }
      }
    }
    /* Check for missing '=' separator. */
    if (strlen((*parameters)->array[index]->name ) ==
        strlen((*parameters)->array[index]->value)   ) {
      errno = EINVAL;
      RMIX_LOG((RMIX_WARN(missing '=' separator in name-value pair)))
      /* Destroy export parameter list. */
      for (index = 0; index < count; index++) {
        /* Destroy export parameter. */
        if (0 != rmix_parameter_destroy(&(*parameters)->array[index])) {
          RMIX_LOG((RMIX_WARN(unable to destroy export parameter.)))
        }
      }
      /* Deallocate export parameter list array. */
      free((*parameters)->array);
      /* Deallocate export parameter list. */
      free(*parameters);
      errno = EINVAL;
      return -1;
    }
    /* Separate value from name-value pair. */
    value = (*parameters)->array[index]->value;
    memcpy(value, name + 1, strlen(name + 1) + 1);
    /* Remove leading '\'s from value. */
    for (; '\0' != value[0]; value++) {
      /* Check for leading '\' in value. */
      if ('\\' == name[0]) {
        /* Remove leading '\' in value. */
        memmove(value, value + 1, strlen(value));
      }
    }
  }
  return 0;
}

/*
 * Destroys an export parameter list.
 *
 * parameters = The export parameter list to destroy (also return).
 * return     = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_parameters_destroy"
int rmix_parameters_destroy (rmix_parameters_t **parameters) {
  int          result, errno2;
  unsigned int index;
#ifdef DEBUG
  /* Check parameters parameter. */
  if (NULL == parameters) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Check parameters parameter. */
  if (NULL == *parameters) {
    return 0;
  }
  /* Preset return value. */
  result = 0;
  errno2 = 0;
  /* Destroy export parameter list. */
  for (index = 0; index < (*parameters)->count; index++) {
    /* Destroy export parameter. */
    if (0 != rmix_parameter_destroy(&(*parameters)->array[index])) {
      errno2 = errno;
      RMIX_LOG((RMIX_WARN(unable to destroy export parameter)))
      result = -1;
    }
  }
  /* Deallocate export parameter list array. */
  if (NULL != (*parameters)->array) {
    free((*parameters)->array);
  }
  /* Deallocate export parameter list. */
  free(*parameters);
  if (0 != errno2) {
    errno = errno2;
  }
  return result;
}

/*
 * Duplicates an export parameter list.
 *
 * parameters1 = The export parameter list to create (return).
 * parameters2 = The export parameter list to duplicate.
 * return      = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_parameters_duplicate"
int rmix_parameters_duplicate (rmix_parameters_t       **parameters1,
                               const rmix_parameters_t  *parameters2) {
#ifdef DEBUG
  /* Check parameters1 parameter. */
  if (NULL == parameters1) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters1 parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check parameters2 parameter. */
  if (NULL == parameters2) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters2 parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Create export parameter list. */
  if (0 != rmix_parameters_create(parameters1, parameters2->count,
             (const rmix_parameter_t**)parameters2->array)) {
    int errno2 = errno;
    RMIX_LOG((RMIX_WARN(unable to create export parameter list)))
    errno = errno2;
    return -1;
  }
  return 0;
}

/*
 * Returns the string form of an export parameter list.
 *
 * pairs      = The export parameter list name-value pairs string. Each pair
 *              must be in the form <name>=<value>. Use a leading '\' for '='
 *              and spaces that are part of name or value and for other '\'s.
 *              For example, 'name1\=\ =\=value1\\ name2\ =value2' means
 *              'name1= ' and '=value1\', and 'name 2' and 'value2'. The name-
 *              value pairs string may contain leading and trailing spaces, and
 *              multiple consecutive separating spaces (return).
 * parameters = The export parameter list to create.
 * return     = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_parameters_string"
int rmix_parameters_string (char                    **pairs,
                            const rmix_parameters_t  *parameters) {
  unsigned int length, index1, index2;
#ifdef DEBUG
  /* Check pairs parameter. */
  if (NULL == pairs) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(pairs parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check parameters parameter. */
  if (NULL == parameters) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Calculate overall string length. */
  for (length = 0, index1 = 0; index1 < parameters->count; index1++) {
    unsigned int length2;
    /* Calculate name length. */
    length2 = strlen(parameters->array[index1]->name) + 1;
    /* Add to overall string length. */
    length += length2;
    /* Adjust overall string length for leading
       '\' for each ' ' and '=' in name. */
    for (index2 = 0; index2 < length2; index2++) {
      if (('=' == parameters->array[index1]->name[index2])||
          (' ' == parameters->array[index1]->name[index2])) {
        length++;
      }
    }
    /* Calculate value length. */
    length2 = strlen(parameters->array[index1]->value);
    /* Add to overall string length. */
    length += length2;
    /* Adjust overall string length for leading
       '\' for each ' ' and '=' in value. */
    for (index2 = 0; index2 < length2; index2++) {
      if (('=' == parameters->array[index1]->value[index2])||
          (' ' == parameters->array[index1]->value[index2])) {
        length++;
      }
    }
  }
  /* Add length for delimiting spaces to overall string length. */
  length += (0 == parameters->count)?0:(parameters->count - 1);
  /* Allocate export parameter string. */
  if (NULL == (*pairs = malloc((length + 1) * sizeof(char*)))) {
    int errno2 = errno;
    RMIX_LOG((RMIX_WARN(unable to allocate export parameter string)))
    errno = errno2;
    return -1;
  }
  /* Copy names and values. */
  for (length = 0, index1 = 0; index1 < parameters->count; index1++) {
    unsigned int length2;
    /* Calculate name length. */
    length2 = strlen(parameters->array[index1]->name);
    /* Copy name. */
    memcpy(*pairs + length, parameters->array[index1]->name, length2);
    /* Insert leading '\' for each ' ' and '='. */
    for (index2 = 0; index2 < length2; index2++) {
      if (('=' == (*pairs)[length + index2])||
          (' ' == (*pairs)[length + index2])) {
        /* Insert leading '\'. */
        memmove(*pairs + length + index2 + 1,
                *pairs + length + index2, length2 - index2);
        (*pairs)[length + index2] = '\\';
        /* Adjust name length. */
        length2++;
      }
    }
    /* Adjust overall length. */
    length += length2;
    /* Insert delimiting '='. */
    (*pairs)[length] = '=';
    length++;
    /* Calculate value length. */
    length2 = strlen(parameters->array[index1]->value);
    /* Copy value. */
    memcpy(*pairs + length, parameters->array[index1]->value, length2);
    /* Insert leading '\' for each ' ' and '='. */
    for (index2 = 0; index2 < length2; index2++) {
      if (('=' == (*pairs)[length + index2])||
          (' ' == (*pairs)[length + index2])) {
        /* Insert leading '\'. */
        memmove(*pairs + length + index2 + 1,
                *pairs + length + index2, length2 - index2);
        (*pairs)[length + index2] = '\\';
        /* Adjust value length. */
        length2++;
      }
    }
    /* Adjust overall length. */
    length += length2;
    /* Check for next name/vaue pair. */
    if (index1 + 1 < parameters->count) {
      /* Insert delimiting ' '. */
      (*pairs)[length] = ' ';
      length++;
    } else {
      /* Insert closing '\0'. */
      (*pairs)[length] = '\0';
    }
  }
  return 0;
}

/*
 * Compares two export parameter lists.
 *
 * diff        = The difference between both export parameter lists (return).
 *               diff > 0 if parameters1 < parameters2,
 *               diff < 0 if parameters1 > parameters2, and
 *               diff = 0 if parameters1 = parameters2.
 * parameters1 = The export parameter list to compare against.
 * parameters2 = The export parameter list to compare with.
 * return      = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_parameters_compare"
int rmix_parameters_compare (int                     *diff,
                             const rmix_parameters_t *parameters1,
                             const rmix_parameters_t *parameters2) {
  unsigned int index;
#ifdef DEBUG
  /* Check diff parameter. */
  if (NULL == diff) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(diff parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check parameters1 parameter. */
  if (NULL == parameters1) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters1 parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check parameters1->array parameter. */
  if ((0 != parameters1->count)&&(NULL == parameters1->array)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters1->array parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check parameters1->count parameter. */
  if ((0 == parameters1->count)&&(NULL != parameters1->array)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters1->count parameter is zero)))
    errno = EINVAL;
    return -1;
  }
  /* Check parameters2 parameter. */
  if (NULL == parameters2) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters2 parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check parameters2->array parameter. */
  if ((0 != parameters2->count)&&(NULL == parameters2->array)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters2->array parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check parameters2->count parameter. */
  if ((0 == parameters2->count)&&(NULL != parameters2->array)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters2->count parameter is zero)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Compare export parameter list sizes. */
  *diff = parameters2->count - parameters1->count;
  /* Compare export parameter list arrays and check for equality. */
  for (index = 0; 0 == *diff && index < parameters1->count; index++) {
    /* Compare export parameters. */
    if (0 != rmix_parameter_compare(diff, parameters1->array[index],
                                          parameters2->array[index])) {
      int errno2 = errno;
      RMIX_LOG((RMIX_WARN(unable to compare export parameters)))
      errno = errno2;
      return -1;
    }
  }
  return 0;
}

/*
 * Searches for and returns the value of the parameter associated to name.
 *
 * parameters = The export parameter list to search.
 * value      = The export parameter value associated to name (return).
 * name       = The export parameter name to search for.
 * return     = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_parameters_search"
int rmix_parameters_search (const rmix_parameters_t  *parameters,
                            const char              **value,
                            const char               *name) {
  unsigned int index;
#ifdef DEBUG
  /* Check parameters parameter. */
  if (NULL == parameters) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check parameters->array parameter. */
  if ((0 != parameters->count)&&(NULL == parameters->array)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters->array parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check parameters->count parameter. */
  if ((0 == parameters->count)&&(NULL != parameters->array)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters->count parameter is zero)))
    errno = EINVAL;
    return -1;
  }
  /* Check value parameter. */
  if (NULL == value) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(value parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check name parameter. */
  if (NULL == name) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(name parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Search for value of parameter associated to name. */
  for (index = 0; index < parameters->count; index++) {
    /* Compare names. */
    if (0 == strcmp(name, parameters->array[index]->name)) {
      /* Return value of parameter associated to name. */
      *value = parameters->array[index]->value;
      return 0;
    }
  }
  /* Unable to find value of parameter associated to name. */
  errno = ESRCH;
  return -1;
}

/*
 * Searches for and replaces the value of the parameter associated to name.
 *
 * parameters = The export parameter list to search.
 * name       = The export parameter name to search for.
 * value      = The export parameter value to replace.
 * return     = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_parameters_replace"
int rmix_parameters_replace (rmix_parameters_t *parameters,
                             const char        *name,
                             const char        *value) {
  char         *value2;
  unsigned int  index;
#ifdef DEBUG
  /* Check parameters parameter. */
  if (NULL == parameters) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check parameters->array parameter. */
  if ((0 != parameters->count)&&(NULL == parameters->array)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters->array parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check parameters->count parameter. */
  if ((0 == parameters->count)&&(NULL != parameters->array)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters->count parameter is zero)))
    errno = EINVAL;
    return -1;
  }
  /* Check name parameter. */
  if (NULL == name) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(name parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check value parameter. */
  if (NULL == value) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(value parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Search for value of parameter associated to name. */
  for (index = 0; index < parameters->count; index++) {
    /* Compare names. */
    if (0 == strcmp(name, parameters->array[index]->name)) {
      /* Duplicate value. */
      if (NULL == (value2 = strdup(value))) {
        int errno2 = errno;
        RMIX_LOG((RMIX_WARN(unable to duplicate value)))
        errno = errno2;
        return -1;
      }
      /* Deallocate value of parameter associated to name. */
      free(parameters->array[index]->value);
      /* Replace value of parameter associated to name. */
      parameters->array[index]->value = value2;
      return 0;
    }
  }
  /* Unable to find value of parameter associated to name. */
  errno = ESRCH;
  return -1;
}

/*
 * Searches for and replaces the value of the parameter associated to name.
 * Inserts the name/value pair if not found.
 *
 * parameters = The export parameter list to search.
 * name       = The export parameter name to search for.
 * value      = The export parameter value to replace.
 * return     = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_parameters_insert"
int rmix_parameters_insert (rmix_parameters_t *parameters,
                            const char        *name,
                            const char        *value) {
  char         *value2;
  void         *array;
  unsigned int  index;
#ifdef DEBUG
  /* Check parameters parameter. */
  if (NULL == parameters) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check parameters->array parameter. */
  if ((0 != parameters->count)&&(NULL == parameters->array)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters->array parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check parameters->count parameter. */
  if ((0 == parameters->count)&&(NULL != parameters->array)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters->count parameter is zero)))
    errno = EINVAL;
    return -1;
  }
  /* Check name parameter. */
  if (NULL == name) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(name parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check value parameter. */
  if (NULL == value) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(value parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Search for value of parameter associated to name. */
  for (index = 0; index < parameters->count; index++) {
    /* Compare names. */
    if (0 == strcmp(name, parameters->array[index]->name)) {
      /* Duplicate value. */
      if (NULL == (value2 = strdup(value))) {
        int errno2 = errno;
        RMIX_LOG((RMIX_WARN(unable to duplicate value)))
        errno = errno2;
        return -1;
      }
      /* Deallocate value of parameter associated to name. */
      free(parameters->array[index]->value);
      /* Replace value of parameter associated to name. */
      parameters->array[index]->value = value2;
      return 0;
    }
  }
  /* Increase parameters count. */
  index = parameters->count;
  parameters->count++;
  /* Reallocate parameters array. */
  if (NULL == (array = realloc(parameters->array,
                               parameters->count *
                        sizeof(parameters->array[0])))) {
    int errno2 = errno;
    RMIX_LOG((RMIX_WARN(unable to reallocate parameters array)))
    /* Decrease parameters array. */
    parameters->count--;
    errno2 = errno;
    return -1;
  }
  parameters->array = array;
  /* Create parameter. */
  if (0 != rmix_parameter_create(&parameters->array[index], name, value)) {
    int errno2 = errno;
    RMIX_LOG((RMIX_WARN(unable to create parameter)))
    /* Decrease parameters count. */
    parameters->count--;
    /* Reallocate parameters array. */
    if (0 == parameters->count) {
      free(parameters->array);
      parameters->array = NULL;
    } else if (NULL == (array = realloc(parameters->array,
                                        parameters->count *
                                 sizeof(parameters->array[0])))) {
      RMIX_LOG((RMIX_WARN(unable to reallocate parameters array)))
      errno = 0;
    } else {
      parameters->array = array;
    }
    errno2 = errno;
    return -1;
  }
  return 0;
}

/*
 * Searches for the value of the parameter associated to name. Inserts the
 * name/value pair only if not found.
 *
 * parameters = The export parameter list to search.
 * name       = The export parameter name to search for.
 * value      = The export parameter value to replace.
 * return     = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_parameters_insert2"
int rmix_parameters_insert2 (rmix_parameters_t *parameters,
                             const char        *name,
                             const char        *value) {
  void         *array;
  unsigned int  index;
#ifdef DEBUG
  /* Check parameters parameter. */
  if (NULL == parameters) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check parameters->array parameter. */
  if ((0 != parameters->count)&&(NULL == parameters->array)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters->array parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check parameters->count parameter. */
  if ((0 == parameters->count)&&(NULL != parameters->array)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters->count parameter is zero)))
    errno = EINVAL;
    return -1;
  }
  /* Check name parameter. */
  if (NULL == name) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(name parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check value parameter. */
  if (NULL == value) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(value parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Search for value of parameter associated to name. */
  for (index = 0; index < parameters->count; index++) {
    /* Compare names. */
    if (0 == strcmp(name, parameters->array[index]->name)) {
      return 0;
    }
  }
  /* Increase parameters count. */
  index = parameters->count;
  parameters->count++;
  /* Reallocate parameters array. */
  if (NULL == (array = realloc(parameters->array,
                               parameters->count *
                        sizeof(parameters->array[0])))) {
    int errno2 = errno;
    RMIX_LOG((RMIX_WARN(unable to reallocate parameters array)))
    /* Decrease parameters array. */
    parameters->count--;
    errno2 = errno;
    return -1;
  }
  parameters->array = array;
  /* Create parameter. */
  if (0 != rmix_parameter_create(&parameters->array[index], name, value)) {
    int errno2 = errno;
    RMIX_LOG((RMIX_WARN(unable to create parameter)))
    /* Decrease parameters count. */
    parameters->count--;
    /* Reallocate parameters array. */
    if (0 == parameters->count) {
      free(parameters->array);
      parameters->array = NULL;
    } else if (NULL == (array = realloc(parameters->array,
                                        parameters->count *
                                 sizeof(parameters->array[0])))) {
      RMIX_LOG((RMIX_WARN(unable to reallocate parameters array)))
      errno = 0;
    } else {
      parameters->array = array;
    }
    errno2 = errno;
    return -1;
  }
  return 0;
}

/*
 * Searches for and removes the name/value pair.
 *
 * parameters = The export parameter list to search.
 * name       = The export parameter name to search for.
 * return     = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_parameters_remove"
int rmix_parameters_remove (rmix_parameters_t *parameters,
                            const char        *name) {
  void         *array;
  unsigned int  index;
#ifdef DEBUG
  /* Check parameters parameter. */
  if (NULL == parameters) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check parameters->array parameter. */
  if ((0 != parameters->count)&&(NULL == parameters->array)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters->array parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check parameters->count parameter. */
  if ((0 == parameters->count)&&(NULL != parameters->array)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters->count parameter is zero)))
    errno = EINVAL;
    return -1;
  }
  /* Check name parameter. */
  if (NULL == name) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(name parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Search for value of parameter associated to name. */
  for (index = 0; index < parameters->count; index++) {
    /* Compare names. */
    if (0 == strcmp(name, parameters->array[index]->name)) {
      /* Destroy parameter. */
      if (0 != rmix_parameter_destroy(&parameters->array[index])) {
        int errno2 = errno;
        RMIX_LOG((RMIX_WARN(unable to destroy parameter)))
        errno2 = errno;
        return -1;
      }
      /* Decrease parameters count. */
      parameters->count--;
      memmove(parameters->array + index,
              parameters->array + index + 1,
             (parameters->count - index) *
       sizeof(parameters->array[0]));
      /* Reallocate parameters array. */
      if (0 == parameters->count) {
        free(parameters->array);
        parameters->array = NULL;
      } else if (NULL == (array = realloc(parameters->array,
                                          parameters->count *
                                   sizeof(parameters->array[0])))) {
        RMIX_LOG((RMIX_WARN(unable to reallocate parameters array)))
        errno = 0;
      } else {
        parameters->array = array;
      }
      return 0;
    }
  }
  /* Unable to find value of parameter associated to name. */
  errno = ESRCH;
  return -1;
}

/*
 * Searches for and compares the value of the parameter associated to name.
 *
 * parameters = The export parameter list to search.
 * diff       = The difference between both values if found (return).
 *              diff > 0 if value < value in parameters,
 *              diff < 0 if value > value in parameters, and
 *              diff = 0 if value = value in parameters.
 * value      = The export parameter value to compare against.
 * name       = The export parameter name to search for.
 * return     = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_parameters_check"
int rmix_parameters_check (const rmix_parameters_t *parameters,
                           int                     *diff,
                           const char              *value,
                           const char              *name) {
  unsigned int index;
#ifdef DEBUG
  /* Check parameters parameter. */
  if (NULL == parameters) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check parameters->array parameter. */
  if ((0 != parameters->count)&&(NULL == parameters->array)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters->array parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check parameters->count parameter. */
  if ((0 == parameters->count)&&(NULL != parameters->array)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(parameters->count parameter is zero)))
    errno = EINVAL;
    return -1;
  }
  /* Check diff parameter. */
  if (NULL == diff) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(diff parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check value parameter. */
  if (NULL == value) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(value parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check name parameter. */
  if (NULL == name) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(name parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Search for value of parameter associated to name. */
  for (index = 0; index < parameters->count; index++) {
    /* Compare names. */
    if (0 == strcmp(name, parameters->array[index]->name)) {
      /* Compare values. */
      *diff = strcmp(value, parameters->array[index]->value);
      return 0;
    }
  }
  /* Unable to find value of parameter associated to name. */
  errno = ESRCH;
  return -1;
}

/******************************************************************************
 *
 * END OF FILE
 *
 ******************************************************************************/
