/******************************************************************************
 *
 * Header file for rmix signature (object method input/output signature) module.
 * Copyright (c) 2002-2006 Oak Ridge National Laboratory.
 *
 * For more information see the following files in the source distribution top-
 * level directory or package data directory (usually /usr/local/share/package):
 *
 * - README    for general package information.
 * - INSTALL   for package install information.
 * - COPYING   for package license information and copying conditions.
 * - AUTHORS   for package authors information.
 * - ChangeLog for package changes information.
 *
 ******************************************************************************/

/* Avoid to include the content of this header file twice. */
#ifndef RMIX_SIGNATURE_H
#define RMIX_SIGNATURE_H

/******************************************************************************
 *
 * Macros
 *
 ******************************************************************************/

/* Signature type character. */
#define RMIX_SIGNATURE_TYPE_char                c
#define RMIX_SIGNATURE_TYPE_signed_char         b
#define RMIX_SIGNATURE_TYPE_int8_t              b
#define RMIX_SIGNATURE_TYPE_unsigned_char       B
#define RMIX_SIGNATURE_TYPE_uint8_t             B
#define RMIX_SIGNATURE_TYPE_short               w
#define RMIX_SIGNATURE_TYPE_int16_t             w
#define RMIX_SIGNATURE_TYPE_signed_short        w
#define RMIX_SIGNATURE_TYPE_uint16_t            W
#define RMIX_SIGNATURE_TYPE_unsigned_short      W
#define RMIX_SIGNATURE_TYPE_int                 i
#define RMIX_SIGNATURE_TYPE_signed_int          i
#define RMIX_SIGNATURE_TYPE_unsigned_int        I
#define RMIX_SIGNATURE_TYPE_long                l
#define RMIX_SIGNATURE_TYPE_signed_long         l
#define RMIX_SIGNATURE_TYPE_int32_t             l
#define RMIX_SIGNATURE_TYPE_unsigned_long       L
#define RMIX_SIGNATURE_TYPE_uint32_t            L
#define RMIX_SIGNATURE_TYPE_long_long           e
#define RMIX_SIGNATURE_TYPE_signed_long_long    e
#define RMIX_SIGNATURE_TYPE_int64_t             e
#define RMIX_SIGNATURE_TYPE_unsigned_long_long  E
#define RMIX_SIGNATURE_TYPE_uint64_t            E
#define RMIX_SIGNATURE_TYPE_float               f
#define RMIX_SIGNATURE_TYPE_double              d
#define RMIX_SIGNATURE_TYPE_long_double         D
#define RMIX_SIGNATURE_TYPE_string              s
#define RMIX_SIGNATURE_TYPE_fixed_array_        a
#define RMIX_SIGNATURE_TYPE_variable_array_     A
#define RMIX_SIGNATURE_TYPE_structure_          o

/* Signature type character (quoted). */
#define RMIX_SIGNATURE_CTYPE_char                'c'
#define RMIX_SIGNATURE_CTYPE_signed_char         'b'
#define RMIX_SIGNATURE_CTYPE_int8_t              'b'
#define RMIX_SIGNATURE_CTYPE_unsigned_char       'B'
#define RMIX_SIGNATURE_CTYPE_uint8_t             'B'
#define RMIX_SIGNATURE_CTYPE_short               'w'
#define RMIX_SIGNATURE_CTYPE_int16_t             'w'
#define RMIX_SIGNATURE_CTYPE_signed_short        'w'
#define RMIX_SIGNATURE_CTYPE_uint16_t            'W'
#define RMIX_SIGNATURE_CTYPE_unsigned_short      'W'
#define RMIX_SIGNATURE_CTYPE_int                 'i'
#define RMIX_SIGNATURE_CTYPE_signed_int          'i'
#define RMIX_SIGNATURE_CTYPE_unsigned_int        'I'
#define RMIX_SIGNATURE_CTYPE_long                'l'
#define RMIX_SIGNATURE_CTYPE_signed_long         'l'
#define RMIX_SIGNATURE_CTYPE_int32_t             'l'
#define RMIX_SIGNATURE_CTYPE_unsigned_long       'L'
#define RMIX_SIGNATURE_CTYPE_uint32_t            'L'
#define RMIX_SIGNATURE_CTYPE_long_long           'e'
#define RMIX_SIGNATURE_CTYPE_signed_long_long    'e'
#define RMIX_SIGNATURE_CTYPE_int64_t             'e'
#define RMIX_SIGNATURE_CTYPE_unsigned_long_long  'E'
#define RMIX_SIGNATURE_CTYPE_uint64_t            'E'
#define RMIX_SIGNATURE_CTYPE_float               'f'
#define RMIX_SIGNATURE_CTYPE_double              'd'
#define RMIX_SIGNATURE_CTYPE_long_double         'D'
#define RMIX_SIGNATURE_CTYPE_string              's'
#define RMIX_SIGNATURE_CTYPE_fixed_array_        'a'
#define RMIX_SIGNATURE_CTYPE_variable_array_     'A'
#define RMIX_SIGNATURE_CTYPE_structure_          'o'

/* Quoting a signature type character. */
#define RMIX_SIGNATURE_CTYPE(type) RMIX_JOIN(RMIX_SIGNATURE_CTYPE_, type)

/* Translating a type to a signature type within a signature definition. */
#define RMIX_SIGNATURE_TYP_(type1, type2)\
          RMIX_JOIN(RMIX_SIGNATURE_TYPE_ ## type1, type2)
#define RMIX_SIGNATURE_TYPE(type1, type2) RMIX_SIGNATURE_TYP_(type1, type2)

/* Initializes a fixed array signature type. */
#define RMIX_SIGNATURE_TYPE_fixed_array(type, size)\
          RMIX_JOIN(RMIX_JOIN(RMIX_SIGNATURE_TYPE_fixed_array_, size),\
                    RMIX_SIGNATURE_TYPE_ ## type)

/* Initializes a variable array signature type. */
#define RMIX_SIGNATURE_TYPE_variable_array(type)\
          RMIX_JOIN(RMIX_SIGNATURE_TYPE_variable_array_,\
                    RMIX_SIGNATURE_TYPE_ ## type)

/* Initializes a struct signature type. */
#define RMIX_SIGNATURE_TYPE_structure(size, members)\
          RMIX_JOIN(RMIX_SIGNATURE_TYPE_structure_, RMIX_JOIN(size, members))

/* Initializes a signature. */
#define RMIX_SIGNATURE(types) RMIX_STRING(types)

/******************************************************************************
 *
 * Includes
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Data Types
 *
 ******************************************************************************/

/******************************************************************************
 *
 * Function Prototypes
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Data
 *
 ******************************************************************************/

/* None at this time. */

#endif /* RMIX_SIGNATURE_H */

/******************************************************************************
 *
 * END OF FILE
 *
 ******************************************************************************/
