/******************************************************************************
 *
 * Header file for rmixregistryd options (option parser) module.
 * Copyright (c) 2002-2006 Oak Ridge National Laboratory.
 *
 * For more information see the following files in the source distribution top-
 * level directory or package data directory (usually /usr/local/share/package):
 *
 * - README    for general package information.
 * - INSTALL   for package install information.
 * - COPYING   for package license information and copying conditions.
 * - AUTHORS   for package authors information.
 * - ChangeLog for package changes information.
 *
 ******************************************************************************/

/* Avoid to include the content of this header file twice. */
#ifndef RMIXREGISTRYD_OPTIONS_H
#define RMIXREGISTRYD_OPTIONS_H

/******************************************************************************
 *
 * Macros
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Includes
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Data Types
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Function Prototypes
 *
 ******************************************************************************/

/*
 * Initializes the option parser.
 *
 * argc   = The argument vector count.
 * argv   = The argument vector (argument[0] is the program name).
 * envp   = The variable vector.
 * return = 0 on success or -1 on any error with errno set appropriately.
 *          May exit with EXIT_SUCCESS if an option requires it and may exit
 *          with EXIT_FAILURE for unrecognized options and missing values.
 */
int rmixregistryd_options_init (int          argc,
                                const char **argv,
                                const char **envp);

/*
 * Finalizes the option parser.
 *
 * return = 0 on success or -1 on any error with errno set appropriately.
 */
int rmixregistryd_options_fini ();

/*
 * Returns the daemon flag.
 *
 * daemon = The daemon flag (return).
 * return = 0 on success or -1 on any error with errno set appropriately.
 */
int rmixregistryd_options_daemon_get (unsigned int *daemon);

/*
 * Gets the minimum thread count of the thread pool.
 *
 * minimum = The minimum thread count (return).
 * return  = 0 on success or -1 on any error with errno set appropriately.
 */
int rmixregistryd_options_pool_minimum_get (unsigned int *minimum);

/*
 * Gets the maximum thread count of the thread pool.
 *
 * maximum = The maximum thread count (return).
 * return  = 0 on success or -1 on any error with errno set appropriately.
 */
int rmixregistryd_options_pool_maximum_get (unsigned int *maximum);

/*
 * Gets the exiting thread timeout of the thread pool.
 *
 * timeout = The exiting thread timeout (return).
 * return  = 0 on success or -1 on any error with errno set appropriately.
 */
int rmixregistryd_options_pool_timeout_get (unsigned int *timeout);

/*
 * Gets the maximum job count of the thread pools job queue.
 *
 * maximum = The maximum job count (return).
 * return  = 0 on success or -1 on any error with errno set appropriately.
 */
int rmixregistryd_options_jobs_maximum_get (unsigned int *maximum);

/*
 * Gets the waiting thread timeout of the thread pools job queue.
 *
 * timeout = The waiting thread timeout (return).
 * return  = 0 on success or -1 on any error with errno set appropriately.
 */
int rmixregistryd_options_jobs_timeout_get (unsigned int *timeout);

/*
 * Gets the export parameters.
 *
 * parameters = The export parameters (return).
 * return     = 0 on success or -1 on any error with errno set appropriately.
 */
int rmixregistryd_options_parameters_get
      (const rmix_parameters_t ***parameters);

/******************************************************************************
 *
 * Data
 *
 ******************************************************************************/

/* None at this time. */

#endif /* RMIXREGISTRYD_OPTIONS_H */

/******************************************************************************
 *
 * END OF FILE
 *
 ******************************************************************************/
