/******************************************************************************
 *
 * Source file for rmixtest (main) module.
 * Copyright (c) 2002-2006 Oak Ridge National Laboratory.
 *
 * For more information see the following files in the source distribution top-
 * level directory or package data directory (usually /usr/local/share/package):
 *
 * - README    for general package information.
 * - INSTALL   for package install information.
 * - COPYING   for package license information and copying conditions.
 * - AUTHORS   for package authors information.
 * - ChangeLog for package changes information.
 *
 ******************************************************************************/

/******************************************************************************
 *
 * Macros
 *
 ******************************************************************************/

#define RMIXTEST_OPTIONS_VERSION RMIX_STRING(\
rmixtest (RMIX test program) %s-%s.%s\n\
Copyright (c) 2002-2006 Oak Ridge National Laboratory.\n\
See COPYING in the source distribution top-level directory or package\n\
data directory (%s) for license information.\n\
Report bugs to %s.\n)

#define RMIXTEST_OPTIONS_HELP "\
Usage:\n\
  rmixtest[.debug] option\n\
Options:\n\
  -h|--help             Print out command line help.\n\
  -v|--version          Print out version information.\n\
  -c|--client [SERVER]  Starts the test program in client mode.\n\
  -s|--server [SERVER]  Starts the test program in server mode.\n\
Option arguments:\n\
  SERVER is a descriptor for the remote object reference of the test server\n\
  in the form: 'PROTOCOL=<potocol> OBJECTID=<object id>' with optional name/\n\
  value pairs for 'HOST=<IP address or name>', 'PORT=<IP port>', 'BINDING=\n\
  <protocol specific binding>' and 'INTERFACES=<interfaces>'. The default for\n\
  SERVER is 'PROTOCOL=RPC OBJECTID=13'. Examples are:\n\
    -  'PROTOCOL=RPC OBJECTID=13'\n\
    -  'PROTOCOL=RPC OBJECTID=13 HOST=192.168.1.1'\n\
    -  'PROTOCOL=RPC OBJECTID=13 HOST=192.168.1.1 PORT=5555'\n\
    -  'PROTOCOL=RPC OBJECTID=13 HOST=192.168.1.1 PORT=5555 BINDING=X'\n\
    -  'HOST=192.168.1.1 PORT=5555'\n\
    -  'HOST=192.168.1.1 PORT=5555 BINDING=X'\n\
"

/******************************************************************************
 *
 * Includes
 *
 ******************************************************************************/

/* Main module header file. */
#include <rmixtest.h>

/******************************************************************************
 *
 * Data Types
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Function Prototypes
 *
 ******************************************************************************/

/*
 * Parses the command line options and looks up a remote object reference at a
 * RMIX test.
 *
 * argc   = The argument vector count.
 * argv   = The argument vector (argument[0] is the program name).
 * envp   = The variable vector.
 * return = EXIT_SUCCESS on success or EXIT_FAILURE on any error.
 */
int main (int    argc,
          char **argv,
          char **envp);

/******************************************************************************
 *
 * Data
 *
 ******************************************************************************/

/******************************************************************************
 *
 * Functions
 *
 ******************************************************************************/

/*
 * Parses the command line options and looks up a remote object reference at a
 * RMIX test.
 *
 * argc   = The argument vector count.
 * argv   = The argument vector (argument[0] is the program name).
 * envp   = The variable vector.
 * return = EXIT_SUCCESS on success or EXIT_FAILURE on any error.
 */
#undef  __FUNC__
#define __FUNC__ "main"
int main (int    argc,
          char **argv,
          char **envp) {
  char *server;
  /* Check for missing arguments. */
  if (1 == argc) {
      fprintf(stderr, "rmixtest: missing option\n");
      fprintf(stderr, "rmixtest: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
  /* Check for -h|--help option. */
  } else if ((0 == strcmp("-h",     argv[1]))||
             (0 == strcmp("--help", argv[1]))) {
    fprintf(stdout, RMIXTEST_OPTIONS_HELP);
    return EXIT_SUCCESS;
  /* Check for -v|--version option. */
  } else if ((0 == strcmp("-v",        argv[1]))||
             (0 == strcmp("--version", argv[1]))) {
    fprintf(stdout, RMIXTEST_OPTIONS_VERSION,
            RMIX_STRING(RMIXTEST_PACKAGE_NAME),
            RMIX_STRING(RMIXTEST_PACKAGE_VERSION),
            RMIX_STRING(RMIXTEST_PACKAGE_RELEASE),
            RMIX_STRING(RMIXTEST_PACKAGE_DATADIR),
            RMIX_STRING(RMIXTEST_PACKAGE_BUGREPORT));
    return EXIT_SUCCESS;
  /* Check for -c|--client option. */
  } else if ((0 == strcmp("-c",       argv[1]))||
             (0 == strcmp("--client", argv[1]))) {
    /* Check for optional option argument. */
    if (3 > argc) {
      server = RMIXTEST_EXPORT;
    } else {
      server = argv[2];
    }
    /* Check for too many option arguments. */
    if (3 < argc) {
      fprintf(stderr, "rmixtest: too many option arguments\n");
      fprintf(stderr, "rmixtest: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Initialize rmix. */
    if (0 != rmix_init(RMIX_THREADS_POOL_MINIMUM, RMIX_THREADS_POOL_MAXIMUM, 0,
                       RMIX_THREADS_JOBS_MAXIMUM, RMIX_THREADS_JOBS_TIMEOUT)) {
      fprintf(stderr, "rmixtest: unable to initialize rmix\n");
      return EXIT_FAILURE;
    }



    /* Finalize rmix. */
    if (0 != rmix_fini()) {
      fprintf(stderr, "rmixtest: unable to finalize rmix\n");
      return EXIT_FAILURE;
    }
    return EXIT_SUCCESS;
  /* Check for -s|--server option. */
  } else if ((0 == strcmp("-s",       argv[1]))||
             (0 == strcmp("--server", argv[1]))) {
    /* Check for optional option argument. */
    if (3 > argc) {
      server = RMIXTEST_EXPORT;
    } else {
      server = argv[2];
    }
    /* Check for too many option arguments. */
    if (3 < argc) {
      fprintf(stderr, "rmixtest: too many option arguments\n");
      fprintf(stderr, "rmixtest: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Initialize rmix. */
    if (0 != rmix_init(RMIX_THREADS_POOL_MINIMUM, RMIX_THREADS_POOL_MAXIMUM, 0,
                       RMIX_THREADS_JOBS_MAXIMUM, RMIX_THREADS_JOBS_TIMEOUT)) {
      fprintf(stderr, "rmixtest: unable to initialize rmix\n");
      return EXIT_FAILURE;
    }



    /* Finalize rmix. */
    if (0 != rmix_fini()) {
      fprintf(stderr, "rmixtest: unable to finalize rmix\n");
      return EXIT_FAILURE;
    }
    return EXIT_SUCCESS;
  /* Check for unknown option. */
  } else {
      fprintf(stderr, "rmixtest: unrecognized option '%s'\n", argv[1]);
      fprintf(stderr, "rmixtest: Try `%s --help` for more information.\n",
              argv[0]);
    return EXIT_FAILURE;
  }
}

/******************************************************************************
 *
 * END OF FILE
 *
 ******************************************************************************/
