/******************************************************************************
 *
 * Source file for rmix-rpcx auth (authentication) module.
 * Copyright (c) 2002-2006 Oak Ridge National Laboratory.
 *
 * For more information see the following files in the source distribution top-
 * level directory or package data directory (usually /usr/local/share/package):
 *
 * - README    for general package information.
 * - INSTALL   for package install information.
 * - COPYING   for package license information and copying conditions.
 * - AUTHORS   for package authors information.
 * - ChangeLog for package changes information.
 *
 ******************************************************************************/

/******************************************************************************
 *
 * Macros
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Includes
 *
 ******************************************************************************/

/* Main module header file. */
#include "rmix-rpcx.h"

/******************************************************************************
 *
 * Data Types
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Data
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Function Prototypes
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Data
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Functions
 *
 ******************************************************************************/



/*
 * Creates an authentication credentials/verifier pair.
 *
 * creflav = The client authentication credential flavor (return).
 * cresize = The client authentication credential body length (return).
 * crebody = The client authentication credential body (return).
 * verflav = The client authentication verifier flavor (return).
 * versize = The client authentication verifier body length (return).
 * verbody = The client authentication verifier body (return).
 * return  = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_rpcx_auth_create"
int rmix_rpcx_auth_create (rmix_rpcx_stream_authflav_t  *creflav,
                           unsigned int                 *cresize,
                           char                        **crebody,
                           rmix_rpcx_stream_authflav_t  *verflav,
                           unsigned int                 *versize,
                           char                        **verbody) {
#ifdef DEBUG
  /* Check cresize parameter. */
  if (NULL == cresize) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(cresize parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check crebody parameter. */
  if (NULL == crebody) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(crebody parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check versize parameter. */
  if (NULL == versize) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(versize parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check verbody parameter. */
  if (NULL == verbody) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(verbody parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* @@ Authentication not implemented yet. @@ */
  *creflav = RMIX_RPCX_STREAM_AUTHFLAV_NONE;
  *crebody = NULL;
  *cresize = 0;
  *verflav = RMIX_RPCX_STREAM_AUTHFLAV_NONE;
  *verbody = NULL;
  *versize = 0;
  /* @@ Authentication not implemented yet. @@ */
  return 0;
}

/*
 * Authenticates a client at a server.
 *
 * creflav  = The client authentication credential flavor.
 * cresize  = The client authentication credential body length (also return).
 * crebody  = The client authentication credential body (also return).
 * verflav  = The authentication verifier flavor (also return).
 * versize  = The authentication verifier body length (also return).
 * verbody  = The authentication verifier body (also return).
 * authstat = The authentication status.
 * return   = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_rpcx_auth_authenticate"
int rmix_rpcx_auth_authenticate (rmix_rpcx_stream_authflav_t   creflav,
                                 unsigned int                 *cresize,
                                 char                        **crebody,
                                 rmix_rpcx_stream_authflav_t  *verflav,
                                 unsigned int                 *versize,
                                 char                        **verbody,
                                 rmix_rpcx_stream_authstat_t  *authstat) {
#ifdef DEBUG
  /* Check cresize parameter. */
  if (NULL == cresize) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(cresize parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check crebody parameter. */
  if (NULL == crebody) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(crebody parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check *crebody parameter. */
  if ((NULL == *crebody)&&(0 != *cresize)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(*crebody parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check *cresize parameter. */
  if ((0 == *cresize)&&(NULL != *crebody)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(*cresize parameter is zero)))
    errno = EINVAL;
    return -1;
  }
  /* Check verflav parameter. */
  if (NULL == verflav) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(verflav parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check versize parameter. */
  if (NULL == versize) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(versize parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check verbody parameter. */
  if (NULL == verbody) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(verbody parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check *verbody parameter. */
  if ((NULL == *verbody)&&(0 != *versize)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(*verbody parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check *versize parameter. */
  if ((0 == *versize)&&(NULL != *verbody)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(*versize parameter is zero)))
    errno = EINVAL;
    return -1;
  }
  /* Check authstat parameter. */
  if (NULL == authstat) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(authstat parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* @@ Authentication not implemented yet. @@ */
  /* Deallocate client authentication credentials. */
  if (NULL != *crebody) {
    free(*crebody);
    *crebody = NULL;
    *cresize = 0;
  }
  /* Deallocate client authentication verifier. */
  if (NULL != *verbody) {
    free(*verbody);
    *verbody = NULL;
    *versize = 0;
  }
  /* Set server authentication verifier. */
  *verbody = NULL;
  *versize = 0;
  *verflav = RMIX_RPCX_STREAM_AUTHFLAV_NONE;
  /* Set authentication status. */
  *authstat = RMIX_RPCX_STREAM_AUTHSTAT_OK;
  /* @@ Authentication not implemented yet. @@ */
  return 0;
}

/*
 * Verifies a server at a client.
 *
 * creflav  = The client authentication credential flavor.
 * cresize  = The client authentication credential body length (also return).
 * crebody  = The client authentication credential body (also return).
 * verflav  = The client authentication verifier flavor.
 * versize  = The client authentication verifier body length (also return).
 * verbody  = The client authentication verifier body (also return).
 * verflav2 = The server authentication verifier flavor.
 * versize2 = The server authentication verifier body length (also return).
 * verbody2 = The server authentication verifier body (also return).
 * return   = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_rpcx_auth_verify"
int rmix_rpcx_auth_verify (rmix_rpcx_stream_authflav_t   creflav,
                           unsigned int                 *cresize,
                           char                        **crebody,
                           rmix_rpcx_stream_authflav_t   verflav,
                           unsigned int                 *versize,
                           char                        **verbody,
                           rmix_rpcx_stream_authflav_t   verflav2,
                           unsigned int                 *versize2,
                           char                        **verbody2) {
#ifdef DEBUG
  /* Check cresize parameter. */
  if (NULL == cresize) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(cresize parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check crebody parameter. */
  if (NULL == crebody) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(crebody parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check *crebody parameter. */
  if ((NULL == *crebody)&&(0 != *cresize)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(*crebody parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check *cresize parameter. */
  if ((0 == *cresize)&&(NULL != *crebody)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(*cresize parameter is zero)))
    errno = EINVAL;
    return -1;
  }
  /* Check versize parameter. */
  if (NULL == versize) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(versize parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check verbody parameter. */
  if (NULL == verbody) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(verbody parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check *verbody parameter. */
  if ((NULL == *verbody)&&(0 != *versize)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(*verbody parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check *versize parameter. */
  if ((0 == *versize)&&(NULL != *verbody)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(*versize parameter is zero)))
    errno = EINVAL;
    return -1;
  }
  /* Check versize2 parameter. */
  if (NULL == versize2) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(versize2 parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check verbody2 parameter. */
  if (NULL == verbody2) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(verbody2 parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check *verbody2 parameter. */
  if ((NULL == *verbody2)&&(0 != *versize2)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(*verbody2 parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check *versize2 parameter. */
  if ((0 == *versize2)&&(NULL != *verbody2)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(*versize2 parameter is zero)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* @@ Authentication not implemented yet. @@ */
  /* Deallocate client authentication credentials. */
  if (NULL != *crebody) {
    free(*crebody);
    *crebody = NULL;
    *cresize = 0;
  }
  /* Deallocate client authentication verifier. */
  if (NULL != *verbody) {
    free(*verbody);
    *verbody = NULL;
    *versize = 0;
  }
  /* Deallocate server authentication verifier. */
  if (NULL != *verbody2) {
    free(*verbody2);
    *verbody2 = NULL;
    *versize2 = 0;
  }
  /* @@ Authentication not implemented yet. @@ */
  return 0;
}

/******************************************************************************
 *
 * END OF FILE
 *
 ******************************************************************************/
